@@ -21,6 +21,7 @@ import           Data.Foldable (toList)
 import qualified Data.IORef as IORef
 import           Data.Maybe (mapMaybe)
 import qualified Data.Text as Text
+import qualified Data.Text.Encoding as Text
 import qualified Data.Vector as Vector
 import qualified Network.HTTP.Client as HTTP.Client
 import qualified Network.HTTP.Client.TLS as TLS
@@ -166,20 +167,18 @@ main = do
 
     let toolUseTest =
             HUnit.testCase "Create message - tool use" do
-                let calculatorTool = Tool.Tool
-                        { Tool.name = "calculator"
-                        , Tool.description = Just "Perform basic arithmetic"
-                        , Tool.input_schema = Tool.InputSchema
-                            { Tool.type_ = "object"
-                            , Tool.properties = Just $ Aeson.object
+                let calculatorTool = Tool.functionTool
+                        "calculator"
+                        (Just "Perform basic arithmetic")
+                        $ Aeson.object
+                            [ "properties" Aeson..= Aeson.object
                                 [ "expression" Aeson..= Aeson.object
                                     [ "type" Aeson..= ("string" :: Text.Text)
                                     , "description" Aeson..= ("Math expression like 2+2" :: Text.Text)
                                     ]
                                 ]
-                            , Tool.required = Just ["expression"]
-                            }
-                        }
+                            , "required" Aeson..= (["expression"] :: [Text.Text])
+                            ]
 
                 Messages.MessageResponse{ stop_reason, content } <-
                     createMessage
@@ -233,6 +232,121 @@ main = do
                 HUnit.assertBool "Should have positive token count"
                     (input_tokens > 0)
 
+    -- Structured outputs tests require beta header
+    let structuredOutputsOptions = V1.defaultClientOptions
+            { V1.apiKey = Text.pack key
+            , V1.anthropicBeta = Just "structured-outputs-2025-11-13"
+            }
+    let V1.Methods{ V1.createMessage = createMessageStructured } =
+            V1.makeMethodsWith clientEnv structuredOutputsOptions
+
+    let jsonOutputsTest =
+            HUnit.testCase "Create message - JSON outputs" do
+                -- Define output schema
+                let outputSchema = Aeson.object
+                        [ "type" Aeson..= ("object" :: Text.Text)
+                        , "properties" Aeson..= Aeson.object
+                            [ "name" Aeson..= Aeson.object
+                                [ "type" Aeson..= ("string" :: Text.Text)
+                                ]
+                            , "age" Aeson..= Aeson.object
+                                [ "type" Aeson..= ("integer" :: Text.Text)
+                                ]
+                            ]
+                        , "required" Aeson..= (["name", "age"] :: [Text.Text])
+                        , "additionalProperties" Aeson..= False
+                        ]
+
+                Messages.MessageResponse{ stop_reason, content } <-
+                    createMessageStructured
+                        Messages._CreateMessage
+                            { Messages.model = model
+                            , Messages.messages =
+                                [ Messages.Message
+                                    { Messages.role = Messages.User
+                                    , Messages.content =
+                                        [ Messages.textContent "Extract: John Smith is 30 years old."
+                                        ]
+                                    , Messages.cache_control = Nothing
+                                    }
+                                ]
+                            , Messages.max_tokens = 200
+                            , Messages.output_format = Just (Messages.jsonSchemaFormat outputSchema)
+                            }
+
+                -- Should complete normally
+                HUnit.assertEqual "Should complete with end_turn"
+                    (Just Messages.End_Turn)
+                    stop_reason
+
+                -- Should have text content that is valid JSON
+                case toList content of
+                    [Messages.ContentBlock_Text{ Messages.text = jsonText }] -> do
+                        case Aeson.eitherDecodeStrict (Text.encodeUtf8 jsonText) of
+                            Left err -> HUnit.assertFailure $ "Invalid JSON: " <> err
+                            Right (obj :: Aeson.Value) -> do
+                                -- Check it has the expected structure
+                                case obj of
+                                    Aeson.Object _ -> pure ()
+                                    _ -> HUnit.assertFailure "Expected JSON object"
+                    _ -> HUnit.assertFailure "Expected single text content block"
+
+    let strictToolUseTest =
+            HUnit.testCase "Create message - strict tool use" do
+                -- Define a tool with strict mode
+                let strictTool = Tool.strictFunctionTool
+                        "get_user"
+                        (Just "Get user by ID")
+                        $ Aeson.object
+                            [ "type" Aeson..= ("object" :: Text.Text)
+                            , "properties" Aeson..= Aeson.object
+                                [ "user_id" Aeson..= Aeson.object
+                                    [ "type" Aeson..= ("integer" :: Text.Text)
+                                    , "description" Aeson..= ("The user ID" :: Text.Text)
+                                    ]
+                                ]
+                            , "required" Aeson..= (["user_id"] :: [Text.Text])
+                            ]
+
+                Messages.MessageResponse{ stop_reason, content } <-
+                    createMessageStructured
+                        Messages._CreateMessage
+                            { Messages.model = model
+                            , Messages.messages =
+                                [ Messages.Message
+                                    { Messages.role = Messages.User
+                                    , Messages.content =
+                                        [ Messages.textContent "Get user 42"
+                                        ]
+                                    , Messages.cache_control = Nothing
+                                    }
+                                ]
+                            , Messages.max_tokens = 200
+                            , Messages.tools = Just [Tool.inlineTool strictTool]
+                            , Messages.tool_choice = Just Tool.ToolChoice_Any
+                            }
+
+                -- Should stop for tool use
+                HUnit.assertEqual "Should stop for tool use"
+                    (Just Messages.Tool_Use)
+                    stop_reason
+
+                -- Should have a tool_use block with integer user_id
+                let toolUseBlocks =
+                        [ (toolId, toolInput)
+                        | Messages.ContentBlock_Tool_Use{ Messages.id = toolId, Messages.input = toolInput }
+                            <- toList content
+                        ]
+                case toolUseBlocks of
+                    [(_, toolInput)] -> do
+                        -- Verify user_id is an integer (strict mode guarantees this)
+                        case Aeson.Types.parseMaybe (Aeson.withObject "input" (\o -> o Aeson..: "user_id")) toolInput of
+                            Just (userId :: Int) ->
+                                HUnit.assertEqual "user_id should be 42" 42 userId
+                            Nothing ->
+                                HUnit.assertFailure "user_id should be present and be an integer"
+                    _ -> HUnit.assertFailure "Expected exactly one tool_use block"
+
     -- Tool search test requires beta header
     let betaOptions = V1.defaultClientOptions
             { V1.apiKey = Text.pack key
@@ -244,47 +358,41 @@ main = do
     let toolSearchTest =
             HUnit.testCase "Create message - tool search" do
                 -- Define several tools to search through
-                let weatherTool = Tool.Tool
-                        { Tool.name = "get_weather"
-                        , Tool.description = Just "Get the current weather for a location"
-                        , Tool.input_schema = Tool.InputSchema
-                            { Tool.type_ = "object"
-                            , Tool.properties = Just $ Aeson.object
+                let weatherTool = Tool.functionTool
+                        "get_weather"
+                        (Just "Get the current weather for a location")
+                        $ Aeson.object
+                            [ "properties" Aeson..= Aeson.object
                                 [ "location" Aeson..= Aeson.object
                                     [ "type" Aeson..= ("string" :: Text.Text)
                                     ]
                                 ]
-                            , Tool.required = Just ["location"]
-                            }
-                        }
+                            , "required" Aeson..= (["location"] :: [Text.Text])
+                            ]
 
-                let stockTool = Tool.Tool
-                        { Tool.name = "get_stock_price"
-                        , Tool.description = Just "Get the stock price for a ticker"
-                        , Tool.input_schema = Tool.InputSchema
-                            { Tool.type_ = "object"
-                            , Tool.properties = Just $ Aeson.object
+                let stockTool = Tool.functionTool
+                        "get_stock_price"
+                        (Just "Get the stock price for a ticker")
+                        $ Aeson.object
+                            [ "properties" Aeson..= Aeson.object
                                 [ "ticker" Aeson..= Aeson.object
                                     [ "type" Aeson..= ("string" :: Text.Text)
                                     ]
                                 ]
-                            , Tool.required = Just ["ticker"]
-                            }
-                        }
+                            , "required" Aeson..= (["ticker"] :: [Text.Text])
+                            ]
 
-                let calculatorTool' = Tool.Tool
-                        { Tool.name = "calculator"
-                        , Tool.description = Just "Perform arithmetic calculations"
-                        , Tool.input_schema = Tool.InputSchema
-                            { Tool.type_ = "object"
-                            , Tool.properties = Just $ Aeson.object
+                let calculatorTool' = Tool.functionTool
+                        "calculator"
+                        (Just "Perform arithmetic calculations")
+                        $ Aeson.object
+                            [ "properties" Aeson..= Aeson.object
                                 [ "expression" Aeson..= Aeson.object
                                     [ "type" Aeson..= ("string" :: Text.Text)
                                     ]
                                 ]
-                            , Tool.required = Just ["expression"]
-                            }
-                        }
+                            , "required" Aeson..= (["expression"] :: [Text.Text])
+                            ]
 
                 -- Use tool search with deferred tools
                 let tools =
@@ -328,19 +436,17 @@ main = do
     let programmaticToolCallingTest =
             HUnit.testCase "Create message - programmatic tool calling" do
                 -- Define a simple tool for PTC
-                let queryTool = Tool.Tool
-                        { Tool.name = "get_data"
-                        , Tool.description = Just "Get data for a key"
-                        , Tool.input_schema = Tool.InputSchema
-                            { Tool.type_ = "object"
-                            , Tool.properties = Just $ Aeson.object
+                let queryTool = Tool.functionTool
+                        "get_data"
+                        (Just "Get data for a key")
+                        $ Aeson.object
+                            [ "properties" Aeson..= Aeson.object
                                 [ "key" Aeson..= Aeson.object
                                     [ "type" Aeson..= ("string" :: Text.Text)
                                     ]
                                 ]
-                            , Tool.required = Just ["key"]
-                            }
-                        }
+                            , "required" Aeson..= (["key"] :: [Text.Text])
+                            ]
 
                 -- Tools: code execution + query tool with allowed_callers
                 let tools =
@@ -457,6 +563,8 @@ main = do
             , messagesConversationTest
             , toolUseTest
             , tokenCountingTest
+            , jsonOutputsTest
+            , strictToolUseTest
             , toolSearchTest
             , programmaticToolCallingTest
             ]
